// hide_weapon_friendly_technique.sma by VEN

#include <amxmodx>
#include <fakemeta>

/*
	  1   (1<<0)  -  crosshair, ammo, weapons list
	  2   (1<<1)  -  flashlight, +
	  4   (1<<2)  -  ALL
	  8   (1<<3)  -  radar, health, armor, +
	 16   (1<<4)  -  timer, +
	 32   (1<<5)  -  money, +
	 64   (1<<6)  -  crosshair
	128   (1<<7)  -  +
*/

// symbol + mean that additional small crosshair will be drawn

#define HIDEHUD_CROSSHAIR_AMMO_WEAPLIST (1<<0) // a
#define HIDEHUD_FLASHLIGHT (1<<1)              // b +
#define HIDEHUD_ALL (1<<2)                     // c
#define HIDEHUD_HEALTH_ARMOR_RADAR (1<<3)      // d +
#define HIDEHUD_ROUND_TIMER (1<<4)             // e +
#define HIDEHUD_MONEY (1<<5)                   // f +
#define HIDEHUD_CROSSHAIR (1<<6)               // g
#define HIDEHUD_NONE (1<<7)                    // h +

#define MAX_CLIENTS 32
new g_hide_hud_elements[MAX_CLIENTS + 1]

new g_msgid_resethud
new g_msgid_hideweapon
new g_msgid_crosshair

new bool:g_skip_msg = false

new g_hide_hud = 0
new g_hide_crosshair = 0

public plugin_init() {
	g_msgid_resethud = get_user_msgid("ResetHUD")
	g_msgid_hideweapon = get_user_msgid("HideWeapon")
	g_msgid_crosshair = get_user_msgid("Crosshair")

	// some hidden hud elements will be redrawn on HUD reset so we use this
	// emessage execution in hook from register_event("XXX", "ev_XXX", "b")
	// causing stack error/recursion (core bug?) so we need this workaround
	register_forward(FM_MessageBegin, "forward_message_begin", 1)
	register_forward(FM_MessageEnd, "forward_message_end", 1) // important: post hook

	register_message(g_msgid_hideweapon, "message_hide_weapon")

	// use this if you going to hide round timer
	//set_msg_block(get_user_msgid("RoundTime"), BLOCK_SET)

	// accepts HIDEHUD flag letters "abcdefgh"
	// see below for HIDEHUD flags/defines list
	register_clcmd("hide", "hide")
}

public hide(id) {
	static arg[10]
	read_argv(1, arg, sizeof arg - 1)
	new hide = read_flags(arg)
	g_hide_hud_elements[id] = hide
	hide_hud_elements(id, hide)

	return PLUGIN_HANDLED
}


public forward_message_begin(msg_dest, msg_id, const Float:origin[3], id) {
	if (!g_skip_msg && msg_dest == MSG_ONE && g_hide_hud_elements[id]) {
		if (msg_id == g_msgid_resethud)
			g_hide_hud = id
		else if (msg_id == g_msgid_hideweapon)
			g_hide_crosshair = id
	}

	return FMRES_IGNORED
}

public forward_message_end() {
	if (g_skip_msg)
		return FMRES_IGNORED

	static id
	if (g_hide_hud) {
		id = g_hide_hud
		g_hide_hud = 0
		if (g_hide_hud_elements[id] && is_user_connected(id))
			hide_hud_elements(id, g_hide_hud_elements[id])
	}
	else if (g_hide_crosshair) {
		id = g_hide_crosshair
		g_hide_crosshair = 0
		if (g_hide_hud_elements[id] && is_user_connected(id))
			hide_crosshair(id)
	}

	return FMRES_IGNORED
}

hide_hud_elements(id, flags) {
	g_skip_msg = true
	emessage_begin(MSG_ONE, g_msgid_hideweapon, _, id)
	ewrite_byte(flags)
	emessage_end()

	hide_crosshair(id)
	g_skip_msg = false
}

public message_hide_weapon(msg_dest, msg_id, id) {
	static hide
	hide = g_hide_hud_elements[id]
	if (hide) {
		static arg
		arg = get_msg_arg_int(1)
		if ((arg & hide) != hide)
			set_msg_arg_int(1, ARG_BYTE, arg | hide)
	}
}

hide_crosshair(id) {
	emessage_begin(MSG_ONE, g_msgid_crosshair, _, id)
	ewrite_byte(0)
	emessage_end()
}

public client_disconnect(id) {
	g_hide_hud_elements[id] = 0
}
